// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Fischer
#include "CountingPrivateSemaphore.hpp"
#include "Fiber.hpp"
#include "Runtime.hpp"
#include "Worker.hpp"
#include "emper.hpp"
#include "fixtures/assert.hpp"

static const unsigned ITERATIONS = 100;
static Runtime* runtime;
static unsigned workerCount;
unsigned iteration = 0;

static void runOn(CPS& cps) {
	ASSERT(Worker::getCurrentWorkerId() == (iteration % workerCount));
	++iteration;
	if (iteration == ITERATIONS) cps.signalAndExit();

	runtime->scheduleOn(*Fiber::from([&] { runOn(cps); }), (iteration % workerCount));
}

static void scheduleOnTest() {
	runtime = Runtime::getRuntime();
	ASSERT(runtime);
	workerCount = runtime->getWorkerCount();

	emper::sleep(1);

	CPS cps(1);
	runtime->scheduleOn(*Fiber::from([&] { runOn(cps); }), (iteration % workerCount));
	cps.wait();
}

void emperTest() { scheduleOnTest(); }
